#pragma once

#include "Component.h"
#include <string>
#include <glm/glm.hpp>



class SpriteComponent : public Component {
public:
    SpriteComponent(Object* owner);

    enum class RenderType {
        Unlit,
        Lit
    };
    

    void SetTexture(const std::string& path);
    void SetNormalMap(const std::string& path);

    unsigned int GetTextureID() const;
    unsigned int GetNormalMapID() const;

    std::string GetTexturePath() const;
    std::string GetNormalMapPath() const;

    bool HasTexture();

    RenderType GetRenderType() const { return renderType; }
    void SetRenderType(RenderType type) { renderType = type; }

    virtual glm::vec2 GetSize() const { return size; }

    virtual std::string GetName() const override { return "SpriteComponent"; }

    virtual void Save(YAML::Emitter& out) const override;
    virtual void Load(const YAML::Node& node) override;

    

private:
    glm::vec2 size = { 64, 64 };
    std::string texturePath;
    std::string normalMapPath;
    RenderType renderType = RenderType::Lit;
    unsigned int textureID = 0;
    unsigned int normalMapID = 0;
    bool texture_loaded = false;
    unsigned int LoadTexture(const std::string& path, bool updateSize);


};
