#pragma once
#include "Component.h"
#include <glm/glm.hpp>
#include <yaml-cpp/yaml.h>
#include <string>
#include <vector>
#include <imgui.h>

class TilemapComponent : public Component
{
public:
    TilemapComponent(Object* owner);

    // Editor UI for tilemap
    void DrawEditorUI();

    // Set up grid and tile dimensions
    void SetGridSize(const glm::ivec2& size);
    void SetTileSize(const glm::ivec2& size);
    void SetTile(int x, int y, int tileIndex);
    int  GetTile(int x, int y) const;
    
    // Getters
    const glm::ivec2& GetGridSize() const { return m_GridSize; }
    const glm::ivec2& GetTileSize() const { return m_TileSize; }
    const std::vector<int>& GetTiles() const { return m_Tiles; }
    const std::string& GetAtlasPath() const { return m_AtlasPath; }
    
    int GetAtlasTileWidth() const { return m_AtlasTileWidth; }
    int GetAtlasTileHeight() const { return m_AtlasTileHeight; }
    int GetAtlasCols() const { return m_AtlasCols; }
    int GetAtlasRows() const { return m_AtlasRows; }

    // Atlas setup: import an atlas image and configure its tile dimensions.
    void SetAtlas(const std::string& path, int atlasTileWidth, int atlasTileHeight);

    // Serialization
    void Save(YAML::Emitter& out) const override;
    void Load(const YAML::Node& node) override;
    std::string GetName() const override { return "TilemapComponent"; }

private:
    // Grid and tile data
    glm::ivec2 m_GridSize {0, 0};
    glm::ivec2 m_TileSize {32, 32};
    std::vector<int> m_Tiles;

    // Atlas (tileset) settings
    std::string m_AtlasPath;
    int m_AtlasTileWidth = 32;
    int m_AtlasTileHeight = 32;
    int m_AtlasCols = 0;
    int m_AtlasRows = 0;

    void UpdateAtlasDimensions();
    bool IsTileFullyTransparent(int col, int row, int texW, int texH, unsigned char* imageData);

};
