#include "EngineConfig.h"
#include <yaml-cpp/yaml.h>
#include <filesystem>
#include <fstream>
#include <Windows.h>
#include <shlobj.h> // SHGetFolderPathA

#include "Logging.h"

EngineConfig g_engineConfig{
    .version = "0.1.0",
    .gl_version = "430",
    .gl_maxLight = 512,
    .settings = {
        .draw_gizmos = true,
        .profile_editor = false,
        .profile_enabled = true,
        .show_color_correction_window = false,
        .lighting_enabled = true,
        .profile_deep = false,
    }};

static std::filesystem::path GetUserSettingsPath()
{
    char userPath[MAX_PATH];
    if (SUCCEEDED(SHGetFolderPathA(NULL, CSIDL_PROFILE, NULL, 0, userPath)))
    {
        std::filesystem::path path = std::filesystem::path(userPath) / ".CreateEngine" / ".user_settings.yaml";
        std::filesystem::create_directories(path.parent_path());
        Logger::LogVerbose("Settings Path: %s", path.string().c_str());

        return path;
    }
    return {};
}

void EngineConfig::SaveToFile()
{
    Logger::LogVerbose("Saving User Settings");

    YAML::Emitter out;
    out << YAML::BeginMap;

    out << YAML::Key << "draw_gizmos" << YAML::Value << settings.draw_gizmos;
    out << YAML::Key << "profile_editor" << YAML::Value << settings.profile_editor;
    out << YAML::Key << "profile_enabled" << YAML::Value << settings.profile_enabled;
    out << YAML::Key << "show_color_correction_window" << YAML::Value << settings.show_color_correction_window;
    out << YAML::Key << "lighting_enabled" << YAML::Value << settings.lighting_enabled;
    out << YAML::Key << "profile_deep" << YAML::Value << settings.profile_deep;


    out << YAML::EndMap;

    std::ofstream fout(GetUserSettingsPath());
    fout << out.c_str();
}

void EngineConfig::LoadFromFile()
{
    Logger::LogVerbose("Loading User Settings");

    auto path = GetUserSettingsPath();
    if (!std::filesystem::exists(path))
        return;

    YAML::Node root = YAML::LoadFile(path.string());

    if (root["draw_gizmos"])
        settings.draw_gizmos = root["draw_gizmos"].as<bool>();
    if (root["profile_editor"])
        settings.profile_editor = root["profile_editor"].as<bool>();
    if (root["profile_enabled"])
        settings.profile_enabled = root["profile_enabled"].as<bool>();
    if (root["show_color_correction_window"])
        settings.show_color_correction_window = root["show_color_correction_window"].as<bool>();
    if (root["lighting_enabled"])
        settings.lighting_enabled = root["lighting_enabled"].as<bool>();
    if (root["profile_deep"])
        settings.profile_deep = root["profile_deep"].as<bool>();
}
