#include "utils.h"
#include <map>
#include "stb_image.h"
#include <windows.h>
#include <filesystem>
#include <string>

static std::map<std::string, GLuint> textureCache;

GLuint LoadTextureIfNeeded(const std::string& path) {
    if (textureCache.count(path))
        return textureCache[path];

    int w, h, channels;
    stbi_set_flip_vertically_on_load(false);
    unsigned char* data = stbi_load(path.c_str(), &w, &h, &channels, 4);
    if (!data)
        return 0;

    GLuint id;
    glGenTextures(1, &id);
    glBindTexture(GL_TEXTURE_2D, id);
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, data);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    stbi_image_free(data);

    textureCache[path] = id;
    return id;
}

std::string GetFilenameFromPath(const std::string& path)
{
    size_t lastSlash = path.find_last_of("/\\");
    std::string filename = (lastSlash == std::string::npos) ? path : path.substr(lastSlash + 1);

    // Strip trailing slashes
    while (!filename.empty() && (filename.back() == '/' || filename.back() == '\\'))
        filename.pop_back();

    return filename;
}




bool DeleteLatestLogFile() {
    // Get %USERPROFILE%
    char userPath[MAX_PATH];
    DWORD len = GetEnvironmentVariableA("USERPROFILE", userPath, MAX_PATH);
    if (len == 0 || len >= MAX_PATH)
        return false;

    std::filesystem::path logPath = std::filesystem::path(userPath) / ".CreateEngine" / ".latest.log";

    if (std::filesystem::exists(logPath)) {
        std::error_code ec;
        std::filesystem::remove(logPath, ec);
        return !ec;  // true = success
    }

    return false;  // File didn't exist
}