#pragma once

#include <string>
#include <GL/glew.h>
#include <glm/glm.hpp>

// Returns filename from a full path (e.g., "C:/foo/bar.png" → "bar.png")
std::string GetFilenameFromPath(const std::string& path);

// Loads a texture (with caching). Returns OpenGL texture ID
GLuint LoadTextureIfNeeded(const std::string& path);

bool DeleteLatestLogFile();


// HSV → RGB helper
inline glm::vec3 HSVtoRGB(float h, float s, float v)
{
    float r, g, b;

    int i = int(h * 6);
    float f = h * 6 - i;
    float p = v * (1 - s);
    float q = v * (1 - f * s);
    float t = v * (1 - (1 - f) * s);

    switch (i % 6)
    {
        case 0: r = v; g = t; b = p; break;
        case 1: r = q; g = v; b = p; break;
        case 2: r = p; g = v; b = t; break;
        case 3: r = p; g = q; b = v; break;
        case 4: r = t; g = p; b = v; break;
        case 5: r = v; g = p; b = q; break;
    }

    return glm::vec3(r, g, b);
}